#ifndef ASSERT
#define ASSERT
#include <assert.h>
#endif

#ifndef COMMONS
#define COMMONS
#include "commons.c"
#endif

#ifndef T_LISTS
#define T_LISTS
#include "t_lists.c"
#endif

#ifndef ARRAYS
#define ARRAYS
#include "arrays.c"
#endif

typedef struct Struct_ArrayList {
	int modCount;
	int size;
	PObject* elementData;
	int elementData_length;
} ArrayList;

typedef ArrayList* PArrayList;

/*------------------------------------------------------
 * Contracts
 *------------------------------------------------------
 */
//@ invariant elementData != null;
bool arraylist_invariant_1(PArrayList arraylist) {
	return (arraylist->elementData != NULL);
}

//@ invariant (\forall int i; size <= i && i < elementData.length ==> elementData[i] == null);
bool arraylist_invariant_2(PArrayList arraylist) {
	int i = arraylist->size;
	for (; i < (arraylist->elementData_length); i++) {
		if (arraylist->elementData[i] != NULL)
			return FALSE;
	}
	return TRUE;
}

//@ invariant 0 <= size && size <= elementData.length;
bool arraylist_invariant_3(PArrayList arraylist) {
	return (0 <= arraylist->size)
			&& (arraylist->size <= arraylist->elementData_length);
}

bool arraylist_invariants(PArrayList arraylist) {
	return arraylist_invariant_1(arraylist) && arraylist_invariant_2(arraylist)
			&& arraylist_invariant_3(arraylist);
}

/*------------------------------------------------------
 * Auxiliary Functions
 *------------------------------------------------------
 */

PObject arraylist_getIndex(PArrayList list, int index) {
	if (index < 0 || index >= list->size)
		return NULL;

	return list->elementData[index];
}

PObject arraylist_setIndex(PArrayList list, int index, PObject new_value) {
	if (index < 0 || index >= list->size)
		return NULL;

	PObject old_value = list->elementData[index];
	list->elementData[index] = new_value;
	return old_value;
}

PObject* arraylist_toArray(PArrayList list, int* new_array_length) {
	PObject* ret_val = arrays_copyOf(list->elementData,
			list->elementData_length, list->size);
	(*new_array_length) = list->size;
	return ret_val;
}

void arraylist_ensureCapacity(PArrayList list, int minCapacity) {
	list->modCount++;
	int oldCapacity = list->elementData_length;
	if (minCapacity > oldCapacity) {
		PObject* oldData = list->elementData;
		int newCapacity = (oldCapacity * 3) / 2 + 1;
		if (newCapacity < minCapacity) {
			newCapacity = minCapacity;
		}

		list->elementData = arrays_copyOf(list->elementData,
				list->elementData_length, newCapacity);
		list->elementData_length = newCapacity;

	}
}

bool arraylist_add(PArrayList list, PObject new_value) {
	arraylist_ensureCapacity(list, list->size + 1);
	list->elementData[list->size++] = new_value;
	return TRUE;
}

/*------------------------------------------------------
 * clear()
 *------------------------------------------------------
 */
void arraylist_clear(PArrayList arraylist) {
	arraylist->modCount++;
	int ic = 0;

	for (; ic < arraylist->size; ic++) {
//		assert (arraylist->elementData!=NULL);	//SAFE:1.411s
//		assert (ic>=0);	//SAFE:1.622s
//		assert (ic<arraylist->elementData_length);	//SAFE:1.622s
		arraylist->elementData[ic] = NULL;
	}
	arraylist->size = 0;
}

void blast_clear(PArrayList arraylist) {
	if (arraylist == NULL)
		return;

	if (!arraylist_invariants(arraylist))
		return;

	int old_size = arraylist->size;
	arraylist_clear(arraylist);

//	assert (arraylist->size==0);	//UNSAFE:1.842s
//	assert (t_lists_eq(arraylist->elementData, 0, old_size, NULL));	//UNSAFE:1.695s
//	assert (t_lists_eq(arraylist->elementData, old_size, arraylist->elementData_length, NULL)); 	//UNSAFE:2.196s

//	assert (arraylist_invariant_1(arraylist));	//SAFE:1.466s
//	assert (arraylist_invariant_2(arraylist));	//UNSAFE:2.112s
//	assert (arraylist_invariant_3(arraylist));	//SAFE:1.751s

}
/*------------------------------------------------------
 * indexOf(Object value)
 *------------------------------------------------------
 */
int arraylist_indexOf(PArrayList arraylist, PObject value) {
	if (value == NULL) {
		int ic = 0;
		for (; ic < arraylist->size; ic++) {
//			assert (arraylist->elementData!=NULL);	//SAFE:0.624s
//			assert (ic>=0);	//SAFE:0.596s
//			assert (ic<arraylist->elementData_length);	//SAFE:0.596s
			if (arraylist->elementData[ic] == NULL) {
				return ic;
			}
		}
	} else {
		int ic = 0;
		for (; ic < arraylist->size; ic++) {
//			assert (arraylist->elementData!=NULL);	//SAFE:0.553s
//			assert (ic>=0);	//SAFE:0.587s
//			assert (ic<arraylist->elementData_length);	//SAFE:0.587s
			if (arraylist->elementData[ic] == value) {
				return ic;
			}
		}
	}
	return -1;
}

void blast_indexOf(PArrayList arraylist, PObject value) {
	if (arraylist == NULL)
		return;

	if (!arraylist_invariants(arraylist))
		return;

	int ret_val = arraylist_indexOf(arraylist, value);

	assert ((ret_val!=-1)==t_lists_in(arraylist->elementData, 0, arraylist->size, value)); 	//UNSAFE:0.288s
//	assert (ret_val==-1 || (0<=ret_val && ret_val<arraylist->size));	//SAFE:0.493s
//	assert (ret_val==-1 || ((arraylist->elementData[ret_val]==value) && !t_lists_in(arraylist->elementData, 0, ret_val, value)));	//UNSAFE:0.289s

//	assert (arraylist_invariant_1(arraylist));	//SAFE:0.477s
//	assert (arraylist_invariant_2(arraylist));	//UNSAFE:1.108s
//	assert (arraylist_invariant_3(arraylist));	//SAFE:0.625s

}
/*------------------------------------------------------
 * lastIndexOf(Object value)
 *------------------------------------------------------
 */
int arraylist_lastIndexOf(PArrayList arraylist, PObject value) {
	if (value == NULL) {
		int ic = arraylist->size - 1;
		for (; ic >= 0; ic--) {
//			assert (arraylist->elementData!=NULL);	//SAFE:0.544s
//			assert (ic>=0);	//SAFE:0.470s
//			assert (ic<arraylist->elementData_length);	//SAFE:0.470s
			if (arraylist->elementData[ic] == NULL) {
				return ic;
			}
		}
	} else {
		int ic = arraylist->size - 1;
		for (; ic >= 0; ic--) {
//			assert (arraylist->elementData!=NULL);	//SAFE:0.633s
//			assert (ic>=0);	//SAFE:0.496s
//			assert (ic<arraylist->elementData_length);	//SAFE:0.496s
			if (arraylist->elementData[ic] == value) {
				return ic;
			}
		}
	}
	return -1;
}

void blast_lastIndexOf(PArrayList arraylist, PObject value) {
	if (arraylist == NULL)
		return;

	if (!arraylist_invariants(arraylist))
		return;

	int ret_val = arraylist_lastIndexOf(arraylist, value);

//	assert ((ret_val==-1)==t_lists_in(arraylist->elementData, 0, arraylist->size, value)); 	//UNSAFE:0.281s
//	assert (ret_val==-1 || (0<=ret_val && ret_val<arraylist->size));	//SAFE:0.460s
//	assert (ret_val==-1 || ((arraylist->elementData[ret_val]==value) && !t_lists_in(arraylist->elementData, ret_val+1, arraylist->size, value)));	//UNSAFE:0.309s

//	assert (arraylist_invariant_1(arraylist));	//SAFE:0.470s
//	assert (arraylist_invariant_2(arraylist));	//UNSAFE:1.095s
//	assert (arraylist_invariant_3(arraylist));	//SAFE:0.616s

}
/*------------------------------------------------------
 * remove0(Object value)
 *------------------------------------------------------
 */
void arraylist_fastRemove(PArrayList arraylist, int index) {
	arraylist->modCount++;
	int numMoved = arraylist->size - index - 1;
	if (numMoved > 0) {
		system_arraycopy(arraylist->elementData, arraylist->elementData_length,
				index + 1, arraylist->elementData,
				arraylist->elementData_length, index, numMoved);
	}
	arraylist->elementData[--arraylist->size] = NULL;
}

bool arraylist_remove0(PArrayList arraylist, PObject value) {
	int index = 0;
	if (value == NULL) {
		for (; index < arraylist->size; index++) {
//			assert (arraylist->elementData!=NULL);	//SAFE:0.581s
//			assert (index>=0);	//SAFE:0.726s
//			assert (index<arraylist->elementData_length);	//SAFE:0.726s
			if (arraylist->elementData[index] == NULL) {
				break;
			}
		}
	} else {
		for (; index < arraylist->size; index++) {
//			assert (arraylist->elementData!=NULL);	//SAFE:0.587s
//			assert (index>=0);	//SAFE:0.587s
//			assert (index<arraylist->elementData_length);	//SAFE:0.587s
			if (arraylist->elementData[index] == value) {
				break;
			}
		}
	}
	if (index < arraylist->size) {
//		assert (0<=index && index<arraylist->size);	//SAFE:0.576s
		arraylist_fastRemove(arraylist, index);
		return TRUE;
	} else {
		return FALSE;
	}
}

void blast_remove0(PArrayList arraylist, PObject value) {
	if (arraylist == NULL)
		return;

	if (!arraylist_invariants(arraylist))
		return;

	int old_size = arraylist->size;
	int old_elementData_length = arraylist->elementData_length;
	PObject* old_elementData = malloc(
			arraylist->elementData_length * sizeof(PObject));
	int i = 0;
	for (; i < arraylist->elementData_length; i++) {
		old_elementData[i] = arraylist->elementData[i];
	}

	bool ret_val = arraylist_remove0(arraylist, value);

//	assert (ret_val==TRUE || arraylist->size==old_size);							//SAFE:0.587s
//	assert (ret_val==TRUE || !t_lists_in(old_elementData, 0, old_size, value));				//UNSAFE:1.601s
//	assert (ret_val==TRUE || t_lists_eq2(arraylist->elementData,old_elementData,0,arraylist->size));	//UNSAFE:2.615s
//	assert (ret_val==FALSE || arraylist->size==old_size -1);						//UNSAFE:1.121s
//	assert (ret_val==FALSE || t_lists_in(old_elementData, 0, old_size, value));				//UNSAFE:1.954s
//
//	assert (arraylist_invariant_1(arraylist));    //SAFE:0.601s
//	assert (arraylist_invariant_2(arraylist));    //UNSAFE:3.554s
//	assert (arraylist_invariant_3(arraylist));    //SAFE:0.907s

}

